@@ -1,14 +1,32 @@
-//-----------------------------------------------------------------------------
-//
-// KDE Display screen saver setup module
-//
-// Copyright (c)  Martin R. Jones 1996,1999,2002
-//
-// Converted to a kcc module by Matthias Hoelzer 1997
-//
-
+/*
+* scrnsave.cpp
+* Copyright 1997       Matthias Hoelzer
+* Copyright 1996,1999,2002    Martin R. Jones
+* Copyright 2004       Chris Howells
+* Copyright 2007-2008  Benjamin Meyer <ben@meyerhome.net>
+* Copyright 2007-2008  Hamish Rodda <rodda@kde.org>
+* Copyright 2009       Dario Andres Rodriguez <andresbajotierra@gmail.com>
+* Copyright 2009       Davide Bettio
+*
+* This program is free software; you can redistribute it and/or
+* modify it under the terms of the GNU General Public License as
+* published by the Free Software Foundation; either version 2 of
+* the License or (at your option) version 3 or any later version
+* accepted by the membership of KDE e.V. (or its successor approved
+* by the membership of KDE e.V.), which shall act as a proxy
+* defined in Section 14 of version 3 of the license.
+*
+* This program is distributed in the hope that it will be useful,
+* but WITHOUT ANY WARRANTY; without even the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* GNU General Public License for more details.
+*
+* You should have received a copy of the GNU General Public License
+* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #include <config-workspace.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
@@ -17,21 +35,13 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 
-#include <kservicetypetrader.h>
-#include <kstandarddirs.h>
 #include <QCheckBox>
-#include <Qt3Support/Q3Header>
 #include <QLabel>
-#include <QLayout>
-#include <Qt3Support/Q3CheckListItem>
 #include <QPushButton>
-#include <QSlider>
 #include <QTimer>
-#include <kmacroexpander.h>
-#include <kshell.h>
-
-//Added by qt3to4:
-#include <QPixmap>
+#include <QTreeWidget>
+#include <QHeaderView>
+#include <QTreeWidgetItem>
 #include <QTextStream>
 #include <QKeyEvent>
 #include <QHBoxLayout>
@@ -39,28 +49,33 @@
 #include <QVBoxLayout>
 #include <QResizeEvent>
 #include <QMouseEvent>
-
-
 #include <QtDBus/QtDBus>
-#include <kaboutdata.h>
-#include <kapplication.h>
-#include <kdebug.h>
-#include <kdialog.h>
-#include <kiconloader.h>
-#include <knuminput.h>
-#include <k3process.h>
-#include <kservicegroup.h>
+
+#include <QDesktopWidget>
+
+#include <kmacroexpander.h>
+#include <KShell>
+#include <KServiceTypeTrader>
+#include <KStandardDirs>
+#include <KApplication>
+#include <KDebug>
+#include <KDialog>
+#include <KAboutData>
+#include <KIcon>
+#include <KNumInput>
+#include <KProcess>
+//#include <KServiceGroup>
+#include <KPluginFactory>
+#include <KPluginLoader>
+
+#include <kscreensaver_interface.h>
+#include <kworkspace/screenpreviewwidget.h>
 
 #include <X11/Xlib.h>
 #include <fixx11h.h>
 
 #include "scrnsave.h"
 #include <QX11Info>
-#include <QDesktopWidget>
-#include <kscreensaver_interface.h>
-#include <KPluginFactory>
-#include <KPluginLoader>
-#include <KMessageBox>
 
 template class QList<SaverConfig*>;
 
@@ -107,186 +122,69 @@ KScreenSaver::KScreenSaver(QWidget *parent, const QVariantList&)
        " the \"Require password\" feature of the screen saver; if you do not, you can still"
        " explicitly lock the session using the desktop's \"Lock Session\" action.</p>"));
 
-    setButtons( KCModule::Help | KCModule::Default | KCModule::Apply );
+    setButtons( KCModule::Help |  KCModule::Apply );
 
 
+    setupUi(this);
 
     readSettings();
 
-    mSetupProc = new K3Process;
-    connect(mSetupProc, SIGNAL(processExited(K3Process *)),
-            this, SLOT(slotSetupDone(K3Process *)));
+    mSetupProc = new KProcess;
+    connect(mSetupProc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(slotSetupDone()));
 
-    mPreviewProc = new K3Process;
-    connect(mPreviewProc, SIGNAL(processExited(K3Process *)),
-            this, SLOT(slotPreviewExited(K3Process *)));
+    mPreviewProc = new KProcess;
+    connect(mPreviewProc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(slotPreviewExited()));
 
-    QBoxLayout *topLayout = new QHBoxLayout(this);
-    topLayout->setMargin(0);
-    topLayout->setSpacing(KDialog::spacingHint());
-
-    // left column
-    QVBoxLayout *leftColumnLayout = new QVBoxLayout( );
-    topLayout->addItem( leftColumnLayout );
-    leftColumnLayout->setSpacing( KDialog::spacingHint() );
-
-    mSaverGroup = new QGroupBox(i18n("Screen Saver"), this );
-    QVBoxLayout *groupLayout = new QVBoxLayout( mSaverGroup );
-    leftColumnLayout->addWidget(mSaverGroup);
-    leftColumnLayout->setStretchFactor( mSaverGroup, 10 );
-
-    mSaverListView = new Q3ListView( mSaverGroup );
-    mSaverListView->setMinimumHeight( 120 );
-    mSaverListView->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
-    mSaverListView->addColumn("");
+    mSaverListView->setColumnCount(1);
     mSaverListView->header()->hide();
     mSelected = -1;
-    groupLayout->addWidget( mSaverListView, 10 );
-    connect( mSaverListView, SIGNAL(doubleClicked ( Q3ListViewItem *)), this, SLOT( slotSetup()));
-    mSaverListView->setWhatsThis( i18n("Select the screen saver to use.") );
+    connect( mSaverListView, SIGNAL(itemDoubleClicked(QTreeWidgetItem*, int)), this, SLOT(slotSetup()));
 
-    QBoxLayout* hlay = new QHBoxLayout();
-    groupLayout->addLayout(hlay);
-    mSetupBt = new QPushButton( i18n("&Setup..."), mSaverGroup );
     connect( mSetupBt, SIGNAL( clicked() ), SLOT( slotSetup() ) );
-    mSetupBt->setEnabled(false);
-    hlay->addWidget( mSetupBt );
-    mSetupBt->setWhatsThis( i18n("Configure the screen saver's options, if any.") );
-
-    mTestBt = new QPushButton( i18n("&Test"), mSaverGroup );
     connect( mTestBt, SIGNAL( clicked() ), SLOT( slotTest() ) );
-    mTestBt->setEnabled(false);
-    hlay->addWidget( mTestBt );
-    mTestBt->setWhatsThis( i18n("Show a full screen preview of the screen saver.") );
 
-    mSettingsGroup = new QGroupBox( i18n("Settings"), this );
-    groupLayout = new QVBoxLayout( mSettingsGroup );
-    leftColumnLayout->addWidget( mSettingsGroup );
-
-    mEnabledCheckBox = new QCheckBox(i18n(
-        "Start a&utomatically"), mSettingsGroup);
     mEnabledCheckBox->setChecked(mEnabled);
-    mEnabledCheckBox->setWhatsThis( i18n(
-        "Automatically start the screen saver after a period of inactivity.") );
     connect(mEnabledCheckBox, SIGNAL(toggled(bool)),
             this, SLOT(slotEnable(bool)));
-    groupLayout->addWidget(mEnabledCheckBox);
 
-    QBoxLayout *hbox = new QHBoxLayout();
-    groupLayout->addLayout(hbox);
-    hbox->addSpacing(30);
-    mActivateLbl = new QLabel(i18n("After:"), mSettingsGroup);
-    mActivateLbl->setEnabled(mEnabled);
-    hbox->addWidget(mActivateLbl);
-    mWaitEdit = new QSpinBox(mSettingsGroup);
-    //mWaitEdit->setSteps(1, 10);
     mWaitEdit->setRange(1, INT_MAX);
-    mWaitEdit->setSuffix(i18n(" min"));
+    mWaitEdit->setSuffix(ki18ncp("unit of time. minutes until the screensaver is triggered",
+                                 " minute", " minutes"));
     mWaitEdit->setValue(mTimeout/60);
     mWaitEdit->setEnabled(mEnabled);
     connect(mWaitEdit, SIGNAL(valueChanged(int)),
             this, SLOT(slotTimeoutChanged(int)));
-    mActivateLbl->setBuddy(mWaitEdit);
-    hbox->addWidget(mWaitEdit);
-    hbox->addStretch(1);
-    QString wtstr = i18n(
-        "The period of inactivity "
-        "after which the screen saver should start.");
-    mActivateLbl->setWhatsThis( wtstr );
-    mWaitEdit->setWhatsThis( wtstr );
 
-    mLockCheckBox = new QCheckBox( i18n(
-        "&Require password to stop"), mSettingsGroup );
     mLockCheckBox->setEnabled( mEnabled );
     mLockCheckBox->setChecked( mLock );
     connect( mLockCheckBox, SIGNAL( toggled( bool ) ),
              this, SLOT( slotLock( bool ) ) );
-    groupLayout->addWidget(mLockCheckBox);
-    mLockCheckBox->setWhatsThis( i18n(
-        "Prevent potential unauthorized use by requiring a password"
-        " to stop the screen saver.") );
-    hbox = new QHBoxLayout();
-    groupLayout->addLayout(hbox);
-    hbox->addSpacing(30);
-    mLockLbl = new QLabel(i18n("After:"), mSettingsGroup);
-    mLockLbl->setEnabled(mEnabled && mLock);
-    mLockLbl->setWhatsThis( i18n(
-        "The amount of time, after the screen saver has started, to ask for the unlock password.") );
-    hbox->addWidget(mLockLbl);
-    mWaitLockEdit = new QSpinBox(mSettingsGroup);
-    //mWaitLockEdit->setSteps(1, 10);
+
     mWaitLockEdit->setRange(1, 300);
-    mWaitLockEdit->setSuffix(i18n(" sec"));
+    mWaitLockEdit->setSuffix(ki18np(" second", " seconds"));
     mWaitLockEdit->setValue(mLockTimeout/1000);
     mWaitLockEdit->setEnabled(mEnabled && mLock);
-    if ( mWaitLockEdit->sizeHint().width() <
-         mWaitEdit->sizeHint().width() ) {
-        mWaitLockEdit->setFixedWidth( mWaitEdit->sizeHint().width() );
-        mWaitEdit->setFixedWidth( mWaitEdit->sizeHint().width() );
-    }
-    else {
-        mWaitEdit->setFixedWidth( mWaitLockEdit->sizeHint().width() );
-        mWaitLockEdit->setFixedWidth( mWaitLockEdit->sizeHint().width() );
-    }
     connect(mWaitLockEdit, SIGNAL(valueChanged(int)),
             this, SLOT(slotLockTimeoutChanged(int)));
-    mLockLbl->setBuddy(mWaitLockEdit);
-    hbox->addWidget(mWaitLockEdit);
-    hbox->addStretch(1);
-    QString wltstr = i18n(
-        "Choose the period "
-        "after which the display will be locked. ");
-    mLockLbl->setWhatsThis( wltstr );
-    mWaitLockEdit->setWhatsThis( wltstr );
 
-    mPlasmaCheckBox = new QCheckBox(i18n("Allow widgets on screen saver"), mSaverGroup);
     mPlasmaCheckBox->setChecked(mPlasmaEnabled);
-    mPlasmaCheckBox->setWhatsThis(i18n("Add widgets to your screensaver"));
     connect(mPlasmaCheckBox, SIGNAL(toggled(bool)), this, SLOT(slotEnablePlasma(bool)));
-    groupLayout->addWidget(mPlasmaCheckBox);
 
-    hbox = new QHBoxLayout();
-    groupLayout->addLayout(hbox);
-    hbox->addSpacing(30);
-    mPlasmaSetup = new QPushButton(i18n("Setup..."), mSaverGroup);
     mPlasmaSetup->setEnabled(mPlasmaEnabled);
     connect(mPlasmaSetup, SIGNAL(clicked()), this, SLOT(slotPlasmaSetup()));
-    hbox->addWidget(mPlasmaSetup);
-    hbox->addItem(new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
 
-    // right column
-    QBoxLayout* rightColumnLayout = new QVBoxLayout();
-    topLayout->addItem( rightColumnLayout );
-    rightColumnLayout->setSpacing( KDialog::spacingHint() );
-
-    mMonitorLabel = new QLabel( this );
-    mMonitorLabel->setAlignment( Qt::AlignCenter );
-    mMonitorLabel->setPixmap( QPixmap(KStandardDirs::locate("data",
-                         "kcontrol/pics/monitor.png")));
-    rightColumnLayout->addWidget(mMonitorLabel, 0);
-    mMonitorLabel->setWhatsThis( i18n("A preview of the selected screen saver.") );
-
-    QBoxLayout* advancedLayout = new QHBoxLayout();
-    rightColumnLayout->addItem( advancedLayout );
-    advancedLayout->setSpacing( 3 );
-    advancedLayout->addWidget( new QWidget( this ) );
-    QPushButton* advancedBt = new QPushButton(
-        i18n( "Advanced &Options" ), this );
-    advancedBt->setObjectName("advancedBtn");
-    advancedBt->setSizePolicy( QSizePolicy(
-        QSizePolicy::Fixed, QSizePolicy::Fixed) );
-    connect( advancedBt, SIGNAL( clicked() ),
-             this, SLOT( slotAdvanced() ) );
-    advancedLayout->addWidget( advancedBt );
-    advancedLayout->addWidget( new QWidget( this ) );
-
-    rightColumnLayout->addStretch();
+    mMonitorPreview = new ScreenPreviewWidget(this);
+    mMonitorPreview->setFixedSize(200,220);
+    QDesktopWidget *desktop = QApplication::desktop();
+    QRect avail = desktop->availableGeometry(desktop->screenNumber(this));
+    mMonitorPreview->setRatio((qreal)avail.width()/(qreal)avail.height());
+    mMonitorPreview->setWhatsThis( i18n("A preview of the selected screen saver.") );
+    mPreviewAreaWidget->layout()->addWidget(mMonitorPreview);
 
     if (mImmutable)
     {
        setButtons(buttons() & ~Default);
        mSettingsGroup->setEnabled(false);
-       mSaverGroup->setEnabled(false);
     }
 
     // finding the savers can take some time, so defer loading until
@@ -317,8 +215,7 @@ void KScreenSaver::resizeEvent( QResizeEvent * )
 
   if (mMonitor)
     {
-      mMonitor->setGeometry( (mMonitorLabel->width()-200)/2+23,
-                 (mMonitorLabel->height()-186)/2+14, 151, 115 );
+      mMonitor->setGeometry( mMonitorPreview->previewRect() );
     }
 }
 
@@ -343,10 +240,13 @@ KScreenSaver::~KScreenSaver()
 {
     if (mPreviewProc)
     {
-        if (mPreviewProc->isRunning())
+        if (mPreviewProc->state() == QProcess::Running)
         {
+            //Avoid triggering slotPreviewExited on close
+            mPreviewProc->disconnect(this);
+
             mPreviewProc->kill( );
-            mPreviewProc->wait( );
+            mPreviewProc->waitForFinished( );
         }
         delete mPreviewProc;
     }
@@ -367,28 +267,17 @@ void KScreenSaver::load()
 //with the following line, the Test and Setup buttons are not enabled correctly
 //if no saver was selected, the "Reset" and the "Enable screensaver", it is only called when starting and when pressing reset, aleXXX
 //    mSelected = -1;
-    int i = 0;
-    Q3ListViewItem *selectedItem = 0;
-	Q_FOREACH( SaverConfig* saver, mSaverList ){
-        if (saver->file() == mSaver)
-        {
-            selectedItem = mSaverListView->findItem ( saver->name(), 0 );
-            if (selectedItem) {
-                mSelected = i;
-                break;
-            }
-        }
-        i++;
-    }
-    if ( selectedItem )
-    {
-      mSaverListView->setSelected( selectedItem, true );
-      mSaverListView->setCurrentItem( selectedItem );
-      slotScreenSaver( selectedItem );
+
+    QTreeWidgetItem * selectedItem = treeItemForSaverFile(mSaver);
+    if (selectedItem) {
+        mSelected = indexForSaverFile(mSaver);
+        mSaverListView->setCurrentItem(selectedItem, QItemSelectionModel::ClearAndSelect);
+        slotScreenSaver(selectedItem);
     }
 
     updateValues();
     mChanged = false;
+
     emit changed(false);
 }
 
@@ -438,12 +327,10 @@ void KScreenSaver::defaults()
     if (mImmutable) return;
 
     slotScreenSaver( 0 );
-
-    Q3ListViewItem *item = mSaverListView->firstChild();
+    QTreeWidgetItem *item = mSaverListView->topLevelItem(0);
     if (item) {
-        mSaverListView->setSelected( item, true );
-        mSaverListView->setCurrentItem( item );
-        mSaverListView->ensureItemVisible( item );
+        mSaverListView->setCurrentItem(item, QItemSelectionModel::ClearAndSelect);
+        mSaverListView->scrollToItem(item);
     }
     slotTimeoutChanged( 5 );
     slotLockTimeoutChanged( 60 );
@@ -488,8 +375,8 @@ void KScreenSaver::save()
 void KScreenSaver::findSavers()
 {
     if ( !mNumLoaded ) {
-	mSaverServices = KServiceTypeTrader::self()->query( "ScreenSaver");
-        new Q3ListViewItem ( mSaverListView, i18n("Loading...") );
+        mSaverServices = KServiceTypeTrader::self()->query( "ScreenSaver");
+        new QTreeWidgetItem ( mSaverListView, QStringList() << i18n("Loading...") );
         if ( mSaverServices.isEmpty() )
             mLoadTimer->stop();
         else
@@ -501,89 +388,112 @@ void KScreenSaver::findSavers()
       SaverConfig *saver = new SaverConfig;
       QString file = KStandardDirs::locate("services", (*it)->entryPath());
       if (saver->read(file)) {
-	      mSaverList.append(saver);
+          mSaverList.append(saver);
         } else
             delete saver;
     }
 
-    if ( mNumLoaded == mSaverServices.count() ) {
-        Q3ListViewItem *selectedItem = 0;
-        int categoryCount = 0;
-        int indx = 0;
-
-        mLoadTimer->stop();
-        delete mLoadTimer;
-		qSort(mSaverList.begin(), mSaverList.end());
-
-        mSelected = -1;
-        mSaverListView->clear();
-        Q_FOREACH( SaverConfig *s, mSaverList )
-        {
-            Q3ListViewItem *item;
-            if (s->category().isEmpty())
-                item = new Q3ListViewItem ( mSaverListView, s->name(), '2' + s->name() );
-            else
-            {
-                Q3ListViewItem *categoryItem = mSaverListView->findItem( s->category(), 0 );
-                if ( !categoryItem ) {
-                    categoryItem = new Q3ListViewItem ( mSaverListView, s->category(), '1' + s->category() );
-                    categoryItem->setPixmap ( 0, SmallIcon ( "preferences-desktop-screensaver" ) );
-                }
-                item = new Q3ListViewItem ( categoryItem, s->name(), s->name() );
-                categoryCount++;
-            }
-            if (s->file() == mSaver) {
-                mSelected = indx;
-                selectedItem = item;
-            }
-            indx++;
-        }
-
-        // Delete categories with only one item
-        Q3ListViewItemIterator it ( mSaverListView );
-        for ( ; it.current(); it++ )
-            if ( it.current()->childCount() == 1 ) {
-               Q3ListViewItem *item = it.current()->firstChild();
-               it.current()->takeItem( item );
-               mSaverListView->insertItem ( item );
-               delete it.current();
-               categoryCount--;
-            }
-
-        mSaverListView->setRootIsDecorated ( categoryCount > 0 );
-        mSaverListView->setSorting ( 1 );
-
-        if ( mSelected > -1 )
-        {
-            mSaverListView->setSelected(selectedItem, true);
-            mSaverListView->setCurrentItem(selectedItem);
-            mSaverListView->ensureItemVisible(selectedItem);
-            mSetupBt->setEnabled(!mSaverList.at(mSelected)->setup().isEmpty());
-            mTestBt->setEnabled(true);
-        }
-
-        connect( mSaverListView, SIGNAL( currentChanged( Q3ListViewItem * ) ),
-                 this, SLOT( slotScreenSaver( Q3ListViewItem * ) ) );
-
-        setMonitor();
+    if ( mNumLoaded != mSaverServices.count() ) {
+        return;
     }
+
+    int categoryCount = 0;
+
+    mLoadTimer->stop();
+    delete mLoadTimer;
+
+    qSort(mSaverList.begin(), mSaverList.end());
+
+    mSaverListView->clear();
+    //Create the treewidget items
+    Q_FOREACH( SaverConfig *s, mSaverList )
+    {
+        QTreeWidgetItem *item = 0;
+        if (s->category().isEmpty()) {
+            //Create top level item without category
+            item = new QTreeWidgetItem ( mSaverListView, QStringList() << s->name() << '2' + s->name() );
+            item->setData(0, Qt::UserRole, s->file());
+        } else {
+            //Create item within a category
+            QList<QTreeWidgetItem*> categoryItemList = mSaverListView->findItems( s->category(), Qt::MatchExactly );
+            QTreeWidgetItem * categoryItem = 0;
+            if ( categoryItemList.isEmpty() ) { //Create the category toplevel item
+                categoryItem = new QTreeWidgetItem ( mSaverListView, QStringList() << s->category() << '1' + s->category() );
+                categoryItem->setIcon ( 0, KIcon ( "preferences-desktop-screensaver" ) );
+            } else {
+                categoryItem = categoryItemList.first();
+            }
+            //Add the child item to the category
+            item = new QTreeWidgetItem ( categoryItem, QStringList() << s->name() << '2' + s->name() );
+            item->setData(0, Qt::UserRole, s->file());
+            categoryCount++;
+        }
+    }
+
+    //Get the current Item and index
+    mSelected = indexForSaverFile(mSaver);
+    QTreeWidgetItem *selectedItem = treeItemForSaverFile(mSaver);
+
+    // Delete categories with only one item
+    QList<QTreeWidgetItem*> itemsToBeDeleted;
+    QList<QTreeWidgetItem*> itemsToBeAddedAsTopLevel;
+    QTreeWidgetItemIterator it(mSaverListView, QTreeWidgetItemIterator::HasChildren);
+    while ((*it)) {
+        if ((*it)->childCount() == 1) {
+            QTreeWidgetItem * item = (*it)->child(0);
+            (*it)->removeChild(item);
+            itemsToBeAddedAsTopLevel.append(item);
+            itemsToBeDeleted.append((*it));
+        }
+        ++it;
+    }
+    mSaverListView->addTopLevelItems(itemsToBeAddedAsTopLevel);
+    qDeleteAll(itemsToBeDeleted);
+
+    mSaverListView->setRootIsDecorated ( categoryCount > 0 );
+    mSaverListView->sortByColumn ( 0, Qt::AscendingOrder );
+
+    //Set the current screensaver
+    if ( mSelected > -1 )
+    {
+        mSaverListView->setCurrentItem(selectedItem, QItemSelectionModel::ClearAndSelect);
+        mSaverListView->scrollToItem(selectedItem);
+
+        mSetupBt->setEnabled(!mSaverList.at(mSelected)->setup().isEmpty());
+        mTestBt->setEnabled(true);
+    }
+
+    connect( mSaverListView, SIGNAL( itemSelectionChanged() ),
+             this, SLOT( slotSelectionChanged() ) );
+
+    setMonitor();
+
 }
 
+void KScreenSaver::slotSelectionChanged()
+{
+    QList<QTreeWidgetItem *> selection = mSaverListView->selectedItems();
+    if (selection.isEmpty()) {
+        slotScreenSaver(0);
+    } else {
+        slotScreenSaver(selection.at(0));
+    }
+}
 //---------------------------------------------------------------------------
 //
 void KScreenSaver::setMonitor()
 {
-    if (mPreviewProc->isRunning())
+    if (mPreviewProc->state() == QProcess::Running)
     // CC: this will automatically cause a "slotPreviewExited"
     // when the viewer exits
     mPreviewProc->kill();
     else
-    slotPreviewExited(mPreviewProc);
+    slotPreviewExited();
 }
 
 //---------------------------------------------------------------------------
 //
-void KScreenSaver::slotPreviewExited(K3Process *)
+void KScreenSaver::slotPreviewExited()
 {
     // Ugly hack to prevent continual respawning of savers that crash
     if (mSelected == mPrevSelected)
@@ -597,20 +507,20 @@ void KScreenSaver::slotPreviewExited(K3Process *)
     // problem).
     delete mMonitor;
 
-    mMonitor = new KSSMonitor(mMonitorLabel);
+    mMonitor = new KSSMonitor(mMonitorPreview);
     QPalette palette;
     palette.setColor(mMonitor->backgroundRole(), Qt::black);
     mMonitor->setPalette(palette);
-    mMonitor->setGeometry((mMonitorLabel->width()-200)/2+23,
-                          (mMonitorLabel->height()-186)/2+14, 151, 115);
+    mMonitor->setGeometry(mMonitorPreview->previewRect());
     mMonitor->show();
     // So that hacks can XSelectInput ButtonPressMask
     XSelectInput(QX11Info::display(), mMonitor->winId(), widgetEventMask );
 
     if (mSelected >= 0) {
-        mPreviewProc->clearArguments();
+        mPreviewProc->clearProgram();
 
         QString saver = mSaverList.at(mSelected)->saver();
+
         QHash<QChar, QString> keyMap;
         keyMap.insert('w', QString::number(mMonitor->winId()));
         *mPreviewProc << KShell::splitArgs(KMacroExpander::expandMacrosShellQuote(saver, keyMap));
@@ -626,10 +536,8 @@ void KScreenSaver::slotPreviewExited(K3Process *)
 void KScreenSaver::slotEnable(bool e)
 {
     mEnabled = e;
-    mActivateLbl->setEnabled( e );
     mWaitEdit->setEnabled( e );
     mLockCheckBox->setEnabled( e );
-    mLockLbl->setEnabled( e && mLock );
     mWaitLockEdit->setEnabled( e && mLock );
     mChanged = true;
     emit changed(true);
@@ -655,31 +563,20 @@ void KScreenSaver::slotPlasmaSetup()
 
 //---------------------------------------------------------------------------
 //
-void KScreenSaver::slotScreenSaver(Q3ListViewItem *item)
+void KScreenSaver::slotScreenSaver(QTreeWidgetItem *item)
 {
     if (!item)
-      return;
-
-    int i = 0, indx = -1;
-	Q_FOREACH( SaverConfig* saver , mSaverList ){
-        if ( item->parent() )
-        {
-            if (  item->parent()->text( 0 ) == saver->category() && saver->name() == item->text (0))
-            {
-                indx = i;
-                break;
-            }
-        }
-        else
-        {
-            if (  saver->name() == item->text (0) )
-            {
-                indx = i;
-                break;
-            }
-        }
-		i++;
+    {
+        mSetupBt->setEnabled(false);
+        mTestBt->setEnabled(false);
+        return;
     }
+
+    //Get the index of the saver file of the current selected treewidget item
+    int indx = indexForSaverFile(item->data(0, Qt::UserRole).toString());
+
+    mSetupBt->setEnabled(item->childCount()==0);
+    mTestBt->setEnabled(item->childCount()==0);
     if (indx == -1) {
         mSelected = -1;
         return;
@@ -687,7 +584,7 @@ void KScreenSaver::slotScreenSaver(Q3ListViewItem *item)
 
     bool bChanged = (indx != mSelected);
 
-    if (!mSetupProc->isRunning())
+    if (mSetupProc->state() != QProcess::Running)
         mSetupBt->setEnabled(!mSaverList.at(indx)->setup().isEmpty());
     mTestBt->setEnabled(true);
     mSaver = mSaverList.at(indx)->file();
@@ -708,10 +605,10 @@ void KScreenSaver::slotSetup()
     if ( mSelected < 0 )
     return;
 
-    if (mSetupProc->isRunning())
+    if (mSetupProc->state() == QProcess::Running)
     return;
 
-    mSetupProc->clearArguments();
+    mSetupProc->clearProgram();
 
     QString saver = mSaverList.at(mSelected)->setup();
     if( saver.isEmpty())
@@ -758,17 +655,6 @@ void KScreenSaver::slotSetup()
     }
 }
 
-//---------------------------------------------------------------------------
-//
-void KScreenSaver::slotAdvanced()
-{
-   KScreenSaverAdvancedDialog dlg( window() );
-   if ( dlg.exec() ) {
-       mChanged = true;
-       emit changed(true);
-  }
-}
-
 //---------------------------------------------------------------------------
 //
 void KScreenSaver::slotTest()
@@ -777,11 +663,11 @@ void KScreenSaver::slotTest()
         return;
 
     if (!mTestProc) {
-        mTestProc = new K3Process;
+        mTestProc = new KProcess;
     } else {
         mPreviewProc->kill();
-        mPreviewProc->wait();
-        mTestProc->clearArguments();
+        mPreviewProc->waitForFinished();
+        mTestProc->clearProgram();
     }
 
     if (!mTestWin)
@@ -808,7 +694,7 @@ void KScreenSaver::slotTest()
     keyMap.insert('w', QString::number(mTestWin->winId()));
     *mTestProc << KShell::splitArgs(KMacroExpander::expandMacrosShellQuote(saver, keyMap));
 
-    mTestProc->start(K3Process::NotifyOnExit);
+    mTestProc->start();
 
     mTesting = true;
 }
@@ -817,9 +703,9 @@ void KScreenSaver::slotTest()
 //
 void KScreenSaver::slotStopTest()
 {
-    if (mTestProc->isRunning()) {
+    if (mTestProc->state() == QProcess::Running) {
         mTestProc->kill();
-        mTestProc->wait();
+        mTestProc->waitForFinished(500);
     }
     releaseMouse();
     releaseKeyboard();
@@ -854,7 +740,6 @@ void KScreenSaver::slotLockTimeoutChanged(int to )
 void KScreenSaver::slotLock( bool l )
 {
     mLock = l;
-    mLockLbl->setEnabled( l );
     mWaitLockEdit->setEnabled( l );
     mChanged = true;
     emit changed(true);
@@ -862,7 +747,7 @@ void KScreenSaver::slotLock( bool l )
 
 //---------------------------------------------------------------------------
 //
-void KScreenSaver::slotSetupDone(K3Process *)
+void KScreenSaver::slotSetupDone()
 {
     mPrevSelected = -1;  // see ugly hack in slotPreviewExited()
     setMonitor();
@@ -870,4 +755,31 @@ void KScreenSaver::slotSetupDone(K3Process *)
     emit changed(true);
 }
 
+QTreeWidgetItem * KScreenSaver::treeItemForSaverFile(const QString & saver)
+{
+    QTreeWidgetItem * item = 0;
+    QTreeWidgetItemIterator it(mSaverListView);
+    while ((*it) && item == 0) {
+        if ((*it)->data(0, Qt::UserRole) == saver) {
+            item = (*it);
+        }
+        ++it;
+    }
+    return item;
+}
+
+int KScreenSaver::indexForSaverFile(const QString & saver)
+{
+    int index = -1;
+    int i = 0;
+    Q_FOREACH( SaverConfig* saverConfig, mSaverList ) {
+        if (saverConfig->file() == saver) {
+            index = i;
+            break;
+        }
+        i++;
+    }
+    return index;
+}
+
 #include "scrnsave.moc"
