@@ -45,6 +45,7 @@ const int LAST_STATE = 6;
 #include <assert.h>
 #include <dirent.h>
 #include <libgen.h>
+#include <math.h>
 
 #include <X11/Xutil.h>
 
@@ -140,6 +141,7 @@ static const char* findFileHelper( const char* name, int* w, int* h, bool locolo
     char best[ 1024 ];
     int best_w = -1;
     int best_h = -1;
+    float best_distance = 10E30;
     DIR* dir = opendir( theme_dir );
     if( dir != NULL )
         {
@@ -150,7 +152,19 @@ static const char* findFileHelper( const char* name, int* w, int* h, bool locolo
                 ? sscanf( file->d_name, "%dx%d-locolor", &w, &h ) == 2
                 : sscanf( file->d_name, "%dx%d", &w, &h ) == 2 )
                 {
-                if( w > best_w
+                // compute difference of areas
+                float delta = w * h - geom.width() * geom.height();
+                // scale down to about 1.0
+                delta /= ((geom.width() * geom.height())+(w * h))/2;
+                // Consider first the difference in aspect ratio,
+                // then in areas. Prefer scaling down.
+                float deltaRatio = 1.0;
+                if (h > 0 && geom.height() > 0) {
+                    deltaRatio = float(w) / float(h) -
+                                 float(geom.width()) / float(geom.height());
+                }
+                float distance = fabs(deltaRatio) * 3.0 + (delta >= 0.0 ? delta : -delta + 5.0);
+                if( distance < best_distance
                     // only derive from themes with the same ratio if lame resolutions are not allowed, damn 1280x1024
                     && ( lame || w * geom.height() == h * geom.width())
                     )
@@ -163,6 +177,7 @@ static const char* findFileHelper( const char* name, int* w, int* h, bool locolo
                         {
                         best_w = w;
                         best_h = h;
+                        best_distance = distance;
                         strcpy( best, tmp );
                         }
                     }
@@ -224,10 +239,14 @@ static const char* findFileWithDepth( const char* name, int* w, int* h, bool loc
         }
     if( w == NULL || h == NULL ) // no scaling possible
         return "";
+#if 0 // disable for now in order to match plasma's selection mechanism
     const char* ret = findFileHelper( name, w, h, locolor, false, geom );
     if( ret == NULL || *ret == '\0' )
         ret = findFileHelper( name, w, h, locolor, true, geom );
     return ret;
+#else
+    return findFileHelper( name, w, h, locolor, true, geom );
+#endif
     }
 
 // returns a pointer to a static !
@@ -335,6 +354,17 @@ static void pregeneratePixmap( const char* file, const char* real_file, int widt
         }
     }
 
+static QImage readImage( FILE* f )
+    {
+    const char jpeg[ 3 + 1 ] = "\377\330\377";
+    char buf[ 3 ] = "";
+    fread( buf, 1, 3, f );
+    rewind( f );
+    if( memcmp( buf, jpeg, 3 ) == 0 )
+        return splash_read_jpeg_image( f );
+    return splash_read_png_image( f );
+    }
+
 static QImage loadImage( const char* file, QRect geom )
     {
     int w, h;
@@ -343,7 +373,7 @@ static QImage loadImage( const char* file, QRect geom )
     FILE* f = fopen( real_file, "r" );
     if( f == NULL )
         return QImage();
-    QImage img = splash_read_png_image( f );
+    QImage img = readImage( f );
     if( img.depth() != 32 )
         img = img.convertDepth( 32 );
     fclose( f );
@@ -396,7 +426,7 @@ static QImage loadAnimImage( const char* file, int frames )
         fprintf( stderr, "Bad anim file: %s\n", file );
         exit( 3 );
         }
-    QImage img = splash_read_png_image( f );
+    QImage img = readImage( f );
     if( img.depth() != 32 )
         img = img.convertDepth( 32 );
     fclose( f );
@@ -573,6 +603,11 @@ static bool waitState( int expected_state )
         close( parent_pipe );
         parent_pipe = -1;
         }
+    const int doubleclick_delay = 200; // mouse doubleclick delay - in ms
+    struct timeval button_press_time, current_time; // we need timeval to deal with milliseconds
+    button_press_time.tv_sec = 0;
+    button_press_time.tv_usec = 0;
+    long click_delay, click_delay_seconds, click_delay_useconds;
     for(;;)
         {
         while( XPending( qt_xdisplay()))
@@ -581,8 +616,20 @@ static bool waitState( int expected_state )
             XNextEvent( qt_xdisplay(), &ev );
             if( ev.type == ButtonPress && ev.xbutton.window == window && ev.xbutton.button == Button1 )
                 {
-                final_time = time( NULL );
-                break;
+                gettimeofday( &current_time, NULL );
+                
+                // find difference in milliseconds with current and previous mouse presses times
+                click_delay_seconds  = current_time.tv_sec  - button_press_time.tv_sec;
+                click_delay_useconds = current_time.tv_usec - button_press_time.tv_usec;
+                click_delay = ( click_delay_seconds * 1000 + click_delay_useconds / 1000.0 ) + 0.5;
+            
+                if( click_delay <= doubleclick_delay )
+                    {
+                    // close splash on doubleclick
+                    final_time = time( NULL );
+                    break;
+                    }
+                gettimeofday( &button_press_time, NULL );
                 }
             if( ev.type == Expose && ev.xexpose.window == window )
                 doPaint( QRect( ev.xexpose.x, ev.xexpose.y, ev.xexpose.width, ev.xexpose.height ));
