@@ -40,7 +40,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 namespace Redmond {
 
-static const char *kdelogo[] = {
+static const char * const kdelogo[] = {
 /* columns rows colors chars-per-pixel */
 "16 16 8 1",
 " 	c None",
@@ -116,20 +116,20 @@ static inline const KDecorationOptions *options()
 static void drawButtonFrame( QPixmap *pix, const QPalette &g, bool sunken )
 {
     QPainter p;
-    int x2 = pix->width() - 1;
-    int y2 = pix->height() - 1;
+    int x2 = pix->width() - 3;
+    int y2 = pix->height() - 3;
     p.begin(pix);
 
     // titlebar button frame
     p.setPen( sunken ? g.color(QPalette::Dark).dark(155) : g.color(QPalette::Light));
-    p.drawLine(0, 0, x2-1, 0);
-    p.drawLine(0, 0, 0, y2-1);
+    p.drawLine(0, 0, x2, 0);
+    p.drawLine(0, 0, 0, y2);
 
     if (sunken)
     {
        p.setPen( g.color(QPalette::Mid).dark(135) );
-       p.drawLine(1, 1, x2-2, 1);
-       p.drawLine(1, 1, 1, y2-2);
+       p.drawLine(1, 1, x2-1, 1);
+       p.drawLine(1, 1, 1, y2-1);
     }
 
     p.setPen( sunken ? g.color(QPalette::Light) : g.color(QPalette::Mid).dark(135));
@@ -174,8 +174,8 @@ static void create_pixmaps ()
     defaultMenuPix = new QPixmap(kdelogo);
 
     // buttons (active/inactive, sunken/unsunken)
-	QColorGroup g = options()->palette(KDecoration::ColorButtonBg, true).active();
-    QColor c = g.background();
+    QPalette g = options()->palette(KDecoration::ColorButtonBg, true);
+    QColor c = g.button().color();
     *btnPix1 = QPixmap(normalTitleHeight, normalTitleHeight-2);
     *btnDownPix1 = QPixmap(normalTitleHeight, normalTitleHeight-2);
     *iBtnPix1 = QPixmap(normalTitleHeight, normalTitleHeight-2);
@@ -193,8 +193,7 @@ static void create_pixmaps ()
         gradientFill(miniBtnDownPix1, c.dark(130), c.light(130));
 
         g = options()->palette(KDecoration::ColorButtonBg, false);
-        g.setCurrentColorGroup( QPalette::Active );
-        c = g.background();
+        c = g.button().color();
         gradientFill(iBtnPix1, c.light(130), c.dark(130));
         gradientFill(iBtnDownPix1, c.dark(130), c.light(130));
         gradientFill(iMiniBtnPix1, c.light(130), c.dark(130));
@@ -206,8 +205,7 @@ static void create_pixmaps ()
         miniBtnDownPix1->fill(c.rgb());
 
         g = options()->palette(KDecoration::ColorButtonBg, false);
-        g.setCurrentColorGroup( QPalette::Active );
-        c = g.background();
+        c = g.button().color();
         iBtnPix1->fill(c.rgb());
         iBtnDownPix1->fill(c.rgb());
         iMiniBtnPix1->fill(c.rgb());
@@ -215,14 +213,12 @@ static void create_pixmaps ()
     }
 
     g = options()->palette(KDecoration::ColorButtonBg, true);
-    g.setCurrentColorGroup( QPalette::Active );
     drawButtonFrame(btnPix1, g, false);
     drawButtonFrame(btnDownPix1, g, true);
     drawButtonFrame(miniBtnPix1, g, false);
     drawButtonFrame(miniBtnDownPix1, g, true);
 
     g = options()->palette(KDecoration::ColorButtonBg, false);
-    g.setCurrentColorGroup( QPalette::Active );
     drawButtonFrame(iBtnPix1, g, false);
     drawButtonFrame(iBtnDownPix1, g, true);
     drawButtonFrame(iMiniBtnPix1, g, false);
@@ -277,7 +273,7 @@ void RedmondButton::reset(unsigned long changed)
 				break;
 			case MenuButton:
 			{
-				QPixmap miniIcon = decoration()->icon().pixmap(QIcon::Small, QIcon::Normal);
+				QPixmap miniIcon = decoration()->icon().pixmap(QSize(16, 16), QIcon::Normal, QIcon::On);
 				if (!miniIcon.isNull()) {
 					setPixmap(miniIcon);
 				} else {
@@ -338,10 +334,15 @@ void RedmondButton::drawButton(QPainter *p)
 				p->drawPixmap(0, 0, miniBtn ? *iMiniBtnPix1 : *iBtnPix1);
 		}
 
-		p->setPen( *btnForeground );
-		int xOff = (width()-10)/2;
-		int yOff = (height()-10)/2;
-		p->drawPixmap(isDown() ? xOff+1: xOff, isDown() ? yOff+1 : yOff, deco);
+		QPainterPath path;
+		path.addRegion( deco );
+		p->setPen( Qt::NoPen );
+		p->setBrush( *btnForeground );
+		QPoint offset( ( width() - 10 )/2, ( height() - 10 )/2 );
+		if( isDown() ) offset += QPoint( 1, 1 );
+		p->translate( offset );
+		p->drawPath( path );
+    
 	} else {
 		if (isLeft() ) {
 			p->fillRect(0, 0, width(), height(),
@@ -416,10 +417,10 @@ int RedmondDeco::layoutMetric(LayoutMetric lm, bool respectWindowState, const KC
 
 		case LM_TitleEdgeLeft:
 		case LM_TitleEdgeRight:
-			return border ? borderWidth+2 : 2;
+			return border ? borderWidth+2 : 0;
 
 		case LM_TitleEdgeTop:
-			return border ? borderWidth+2 : 2;
+			return border ? borderWidth+2 : 0;
 
 		case LM_TitleEdgeBottom:
 			return border ? 1 : 0;
@@ -502,7 +503,7 @@ void RedmondDeco::paintEvent( QPaintEvent* )
     int x2 = r.width()-1;
     int y2 = r.height()-1;
     int w  = r.width();
-    int h  = r.height();
+    //int h  = r.height();
 
     // Draw part of the frame that is the frame color
     // ==============================================
