@@ -141,17 +141,39 @@ void Workspace::setupCompositing()
           break; // don't fall through (this is a testing one) */
 #ifdef KWIN_HAVE_OPENGL_COMPOSITING
         case OpenGLCompositing:
-            kDebug( 1212 ) << "OpenGL compositing";
-            scene = new SceneOpenGL( this );
-            if( !scene->initFailed())
-                break; // -->
-            delete scene;
-            scene = NULL;
-            break; // do not fall back to XRender for now, maybe in the future
+            {
+            kDebug( 1212 ) << "Initializing OpenGL compositing";
+
+            // Some broken drivers crash on glXQuery() so to prevent constant KWin crashes:
+            KSharedConfigPtr unsafeConfigPtr( KSharedConfig::openConfig( "kwinrc" ));
+            KConfigGroup unsafeConfig( unsafeConfigPtr, "Compositing" );
+            if( unsafeConfig.readEntry( "OpenGLIsUnsafe", false ))
+                kWarning( 1212 ) << "KWin has detected that your OpenGL library is unsafe to use, "
+                                    "falling back to XRender.";
+            else
+                {
+                unsafeConfig.writeEntry( "OpenGLIsUnsafe", true );
+                unsafeConfig.sync();
+
+                scene = new SceneOpenGL( this );
+
+                // TODO: Add 30 second delay to protect against screen freezes as well
+                unsafeConfig.writeEntry( "OpenGLIsUnsafe", false );
+                unsafeConfig.sync();
+
+                if( !scene->initFailed())
+                    break; // -->
+                delete scene;
+                scene = NULL;
+                }
+
+            // Do not Fall back to XRender - it causes problems when selfcheck fails during startup, but works later on
+            break;
+            }
 #endif
 #ifdef KWIN_HAVE_XRENDER_COMPOSITING
         case XRenderCompositing:
-            kDebug( 1212 ) << "XRender compositing";
+            kDebug( 1212 ) << "Initializing XRender compositing";
             scene = new SceneXrender( this );
           break;
 #endif
@@ -159,7 +181,7 @@ void Workspace::setupCompositing()
 #ifndef KWIN_HAVE_COMPOSITING
             kDebug( 1212 ) << "Compositing was not available at compile time";
 #else
-            kDebug( 1212 ) << "No compositing";
+            kDebug( 1212 ) << "No compositing enabled";
 #endif
             delete cm_selection;
           return;
@@ -272,6 +294,14 @@ void Workspace::finishCompositing()
 #endif
     }
 
+// OpenGL self-check failed, fallback to XRender
+void Workspace::fallbackToXRenderCompositing()
+    {
+    finishCompositing();
+    options->compositingMode = XRenderCompositing;
+    setupCompositing();
+    }
+
 void Workspace::lostCMSelection()
     {
     kDebug( 1212 ) << "Lost compositing manager selection";
@@ -297,7 +327,7 @@ void Workspace::toggleCompositing()
         if( !shortcut.isEmpty() && options->useCompositing )
             {
             // display notification only if there is the shortcut
-            message = i18n( "Compositing has been suspended by another application.<br/>"
+            message = i18n( "Desktop effects have been suspended by another application.<br/>"
                 "You can resume using the '%1' shortcut.", shortcut );
             Notify::raise( Notify::CompositingSuspendedDbus, message );
             }
@@ -314,6 +344,9 @@ void Workspace::suspendCompositing( bool suspend )
     compositingSuspended = suspend;
     finishCompositing();
     setupCompositing(); // will do nothing if suspended
+    // notify decorations that composition state has changed
+    KDecorationFactory* factory = mgr->factory();
+    factory->reset(SettingCompositing);
     emit compositingToggled( !compositingSuspended );
     }
 
@@ -548,12 +581,12 @@ void Workspace::checkCompositePaintTime( int msec )
         if( KAction* action = qobject_cast<KAction*>( keys->action("Suspend Compositing")))
             shortcut = action->globalShortcut().primary().toString(QKeySequence::NativeText);
         if (shortcut.isEmpty())
-            message = i18n( "Compositing was too slow and has been suspended.\n"
-                "You can disable functionality checks in advanced compositing settings." );
+            message = i18n( "Desktop effects were too slow and have been suspended.\n"
+                "You can disable functionality checks in System Settings (on the Advanced tab in Desktop Effects)." );
         else
-            message = i18n( "Compositing was too slow and has been suspended.\n"
+            message = i18n( "Desktop effects were too slow and have been suspended.\n"
                 "If this was only a temporary problem, you can resume using the '%1' shortcut.\n"
-                "You can also disable functionality checks in advanced compositing settings.", shortcut );
+                "You can disable functionality checks in System Settings (on the Advanced tab in Desktop Effects).", shortcut );
         Notify::raise( Notify::CompositingSlow, message );
         compositeTimer.start( 1000 ); // so that it doesn't trigger sooner than suspendCompositing()
         }
@@ -755,6 +788,10 @@ Pixmap Toplevel::createWindowPixmap()
     }
 
 #ifdef HAVE_XDAMAGE
+// We must specify that the two events are a union so the compiler doesn't
+// complain about strict aliasing rules.
+typedef union { XEvent e; XDamageNotifyEvent de; } EventUnion;
+
 void Toplevel::damageNotifyEvent( XDamageNotifyEvent* e )
     {
     QRegion damage( e->area.x, e->area.y, e->area.width, e->area.height );
@@ -762,11 +799,11 @@ void Toplevel::damageNotifyEvent( XDamageNotifyEvent* e )
     int cnt = 1;
     while( XPending( display()))
         {
-        XEvent e2;
-        if( XPeekEvent( display(), &e2 ) && e2.type == Extensions::damageNotifyEvent()
-            && e2.xany.window == frameId())
+        EventUnion e2;
+        if( XPeekEvent( display(), &e2.e ) && e2.e.type == Extensions::damageNotifyEvent()
+            && e2.e.xany.window == frameId())
             {
-            XNextEvent( display(), &e2 );
+            XNextEvent( display(), &e2.e );
             if( cnt > 200 )
                 {
                 // If there are way too many damage events in the queue, just discard them
@@ -776,8 +813,7 @@ void Toplevel::damageNotifyEvent( XDamageNotifyEvent* e )
                 damage = rect();
                 continue;
                 }
-            XDamageNotifyEvent* e = reinterpret_cast< XDamageNotifyEvent* >( &e2 );
-            QRect r( e->area.x, e->area.y, e->area.width, e->area.height );
+            QRect r( e2.de.area.x, e2.de.area.y, e2.de.area.width, e2.de.area.height );
             ++cnt;
             // If there are too many damaged rectangles, increase them
             // to be multiples of 100x100 px grid, since QRegion get quite
