@@ -147,10 +147,8 @@ unsigned long KDecorationOptionsPrivate::updateSettings( KConfig* config )
     inactiveFont = wmConfig.readEntry("inactiveFont", activeFont);
 
     activeFontSmall = activeFont;
-#ifdef __GNUC__
-#warning KDE4 : is it useful ? ( temporary hack )
-#endif    
-//    activeFontSmall.setPointSize(activeFont.pointSize() - 2 > 0 ? activeFont.pointSize() - 2 : activeFont.pointSize()+1 );
+    // TODO: Is it useful ? (Temporary hack)
+    //activeFontSmall.setPointSize(activeFont.pointSize() - 2 > 0 ? activeFont.pointSize() - 2 : activeFont.pointSize()+1 );
     activeFontSmall = wmConfig.readEntry("activeFontSmall", activeFontSmall);
     inactiveFontSmall = wmConfig.readEntry("inactiveFontSmall", activeFontSmall);
 
