@@ -22,16 +22,30 @@ void BenchmarkSignalPlotter::addData()
     s->addBeam(Qt::red);
     s->addBeam(Qt::yellow);
     s->show();
+    s->setMaxAxisTextWidth(5);
     s->resize(1000,500);
-    s->setMaxAxisTextWidth(50);
     QTest::qWaitForWindowShown(s);
 
     QBENCHMARK {
-        s->addSample(QList<double>() << qrand()%10 << qrand()%10 << qrand()%10 << qrand()%10);
+        s->addSample(QList<qreal>() << qrand()%10 << qrand()%10 << qrand()%10 << qrand()%10);
         qApp->processEvents();
-//        s->repaint();
     }
 
 }
+
+void BenchmarkSignalPlotter::addDataWhenHidden()
+{
+    s->addBeam(Qt::blue);
+    s->addBeam(Qt::green);
+    s->addBeam(Qt::red);
+    s->addBeam(Qt::yellow);
+
+    QBENCHMARK {
+        s->addSample(QList<qreal>() << qrand()%10 << qrand()%10 << qrand()%10 << qrand()%10);
+        qApp->processEvents();
+    }
+
+}
+
 QTEST_KDEMAIN(BenchmarkSignalPlotter, GUI)
 
