@@ -24,72 +24,24 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 // Own
 #include "task.h"
+#include "task_p.h"
 
 // Qt
 #include <QMimeData>
-#include <QTime>
 #include <QTimer>
 #include <QApplication>
 #include <QDesktopWidget>
 
-#ifdef Q_WS_X11
-#include <QX11Info>
-#endif
-
 // KDE
 #include <KDebug>
 #include <KIconLoader>
 #include <KLocale>
-#include <NETWinInfo>
 
 #include "taskmanager.h"
 
 namespace TaskManager
 {
 
-static const unsigned long windowInfoFlags = NET::WMState | NET::XAWMState | NET::WMDesktop |
-                                             NET::WMVisibleName | NET::WMGeometry |
-                                             NET::WMWindowType | NET::WM2AllowedActions;
-static const unsigned long windowInfoFlags2 = NET::WM2AllowedActions;
-
-class Task::Private
-{
-public:
-    Private(WId w)
-     : active(false),
-       win(w),
-       frameId(w),
-       info(KWindowSystem::windowInfo(w, windowInfoFlags, windowInfoFlags2)),
-       lastWidth(0),
-       lastHeight(0),
-       lastResize(false),
-       lastIcon(),
-       cachedChanges(0),
-       cachedChangesTimerId(0)
-    {
-    }
-
-    bool active;
-    WId win;
-    WId frameId;
-    QPixmap pixmap;
-    KWindowInfo info;
-    WindowList transients;
-    WindowList transientsDemandingAttention;
-
-    int lastWidth;
-    int lastHeight;
-    bool lastResize;
-    QPixmap lastIcon;
-    QIcon icon;
-
-    QRect iconGeometry;
-
-    QTime lastUpdate;
-    unsigned int cachedChanges;
-    int cachedChangesTimerId;
-};
-
 Task::Task(WId w, QObject *parent, const char *name)
   : QObject(parent),
     d(new Private(w))
@@ -97,21 +49,7 @@ Task::Task(WId w, QObject *parent, const char *name)
     setObjectName(name);
 
     // try to load icon via net_wm
-    d->pixmap = KWindowSystem::icon(d->win, 16, 16, true);
-
-    // try to guess the icon from the classhint
-    if (d->pixmap.isNull()) {
-        KIconLoader::global()->loadIcon(className().toLower(),
-                                                    KIconLoader::Small,
-                                                    KIconLoader::Small,
-                                                    KIconLoader::DefaultState,
-                                                    QStringList(), 0, true);
-    }
-
-    // load the icon for X applications
-    if (d->pixmap.isNull()) {
-        d->pixmap = SmallIcon("xorg");
-    }
+    refreshIcon();
 }
 
 Task::~Task()
@@ -121,10 +59,11 @@ Task::~Task()
 
 void Task::timerEvent(QTimerEvent *)
 {
-    if (d->cachedChanges) {
+    if (d->cachedChanges.netWindowInfoProperties || d->cachedChanges.netWindowInfoProperties2) {
         d->lastUpdate = QTime();
         refresh(d->cachedChanges);
-        d->cachedChanges = 0;
+        d->cachedChanges.netWindowInfoProperties = 0;
+        d->cachedChanges.netWindowInfoProperties2 = 0;
     }
 
     killTimer(d->cachedChangesTimerId);
@@ -139,10 +78,10 @@ void Task::refreshIcon()
     // try to guess the icon from the classhint
     if (d->pixmap.isNull()) {
         d->pixmap = KIconLoader::global()->loadIcon(className().toLower(),
-                                                    KIconLoader::Small,
-                                                    KIconLoader::Small,
-                                                    KIconLoader::DefaultState,
-                                                    QStringList(), 0, true);
+                                                     KIconLoader::Small,
+                                                     KIconLoader::Small,
+                                                     KIconLoader::DefaultState,
+                                                     QStringList(), 0, true);
 
         // load the icon for X applications
         if (d->pixmap.isNull()) {
@@ -155,10 +94,11 @@ void Task::refreshIcon()
     emit changed(IconChanged);
 }
 
-::TaskManager::TaskChanges Task::refresh(unsigned int dirty)
+::TaskManager::TaskChanges Task::refresh(WindowProperties dirty)
 {
     if (!d->lastUpdate.isNull() && d->lastUpdate.elapsed() < 200) {
-        d->cachedChanges |= dirty;
+        d->cachedChanges.netWindowInfoProperties |= dirty.netWindowInfoProperties;
+        d->cachedChanges.netWindowInfoProperties2 |= dirty.netWindowInfoProperties2;
 
         if (!d->cachedChangesTimerId) {
             d->cachedChangesTimerId = startTimer(200 - d->lastUpdate.elapsed());
@@ -179,27 +119,27 @@ void Task::refreshIcon()
 
     d->info = info;
 
-    if (dirty & NET::WMState || dirty & NET::XAWMState) {
+    if (dirty.netWindowInfoProperties & NET::WMState || dirty.netWindowInfoProperties & NET::XAWMState) {
         changes |= StateChanged;
     }
 
-    if (dirty & NET::WMDesktop) {
+    if (dirty.netWindowInfoProperties & NET::WMDesktop) {
         changes |= DesktopChanged;
     }
 
-    if (dirty & NET::WMGeometry) {
+    if (dirty.netWindowInfoProperties & NET::WMGeometry) {
         changes |= GeometryChanged;
     }
 
-    if (dirty & NET::WMWindowType) {
+    if (dirty.netWindowInfoProperties & NET::WMWindowType) {
         changes |= WindowTypeChanged;
     }
 
-    if (dirty & NET::WM2AllowedActions) {
+    if (dirty.netWindowInfoProperties2 & NET::WM2AllowedActions) {
         changes |= ActionsChanged;
     }
 
-    if (dirty & NET::WMIcon) {
+    if (dirty.netWindowInfoProperties & NET::WMIcon) {
         refreshIcon();
     }
 
@@ -288,6 +228,10 @@ bool Task::isModified() const
 
 int Task::desktop() const
 {
+    if (KWindowSystem::numberOfDesktops() < 2) {
+        return 0;
+    }
+
     return d->info.desktop();
 }
 
@@ -317,30 +261,6 @@ QRect Task::geometry() const
     return d->info.geometry();
 }
 
-void Task::updateDemandsAttentionState( WId w )
-{
-    if (window() != w) {
-        // 'w' is a transient for this task
-        NETWinInfo i( QX11Info::display(), w, QX11Info::appRootWindow(), NET::WMState );
-        if (i.state() & NET::DemandsAttention) {
-            if (!d->transientsDemandingAttention.contains(w)) {
-                d->transientsDemandingAttention.insert(w);
-            }
-        } else {
-            d->transientsDemandingAttention.remove(w);
-        }
-    }
-}
-
-void Task::addTransient( WId w, const NETWinInfo& info )
-{
-    d->transients.insert(w);
-    if (info.state() & NET::DemandsAttention) {
-        d->transientsDemandingAttention.insert(w);
-        emit changed(TransientsChanged);
-    }
-}
-
 void Task::removeTransient(WId w)
 {
     d->transients.remove(w);
@@ -377,44 +297,21 @@ QString Task::name() const
     return d->info.name();
 }
 
-QString Task::className() const
-{
-    XClassHint hint;
-    if(XGetClassHint(QX11Info::display(), d->win, &hint)) {
-        QString nh( hint.res_name );
-        XFree( hint.res_name );
-        XFree( hint.res_class );
-        return nh;
-    }
-    return QString();
-}
-
-QString Task::classClass() const
-{
-    XClassHint hint;
-    if(XGetClassHint(QX11Info::display(), d->win, &hint)) {
-        QString ch( hint.res_class );
-        XFree( hint.res_name );
-        XFree( hint.res_class );
-        return ch;
-    }
-    return QString();
-}
-
 QPixmap Task::icon( int width, int height, bool allowResize )
 {
-  if ( (width == d->lastWidth)
-       && (height == d->lastHeight)
-       && (allowResize == d->lastResize )
-       && (!d->lastIcon.isNull()) )
-    return d->lastIcon;
+  if (width == d->lastWidth &&
+      height == d->lastHeight &&
+      allowResize == d->lastResize &&
+      !d->lastIcon.isNull()) {
+      return d->lastIcon;
+  }
 
   QPixmap newIcon = KWindowSystem::icon( d->win, width, height, allowResize );
-  if ( !newIcon.isNull() ) {
-    d->lastIcon = newIcon;
-    d->lastWidth = width;
-    d->lastHeight = height;
-    d->lastResize = allowResize;
+  if (!newIcon.isNull()) {
+      d->lastIcon = newIcon;
+      d->lastWidth = width;
+      d->lastHeight = height;
+      d->lastResize = allowResize;
   }
 
   return newIcon;
@@ -422,13 +319,12 @@ QPixmap Task::icon( int width, int height, bool allowResize )
 
 QIcon Task::icon()
 {
-    if ( !d->icon.isNull() )
-        return d->icon;
-
-    d->icon.addPixmap(KWindowSystem::icon( d->win, KIconLoader::SizeSmall, KIconLoader::SizeSmall, false));
-    d->icon.addPixmap(KWindowSystem::icon( d->win, KIconLoader::SizeSmallMedium, KIconLoader::SizeSmallMedium, false));
-    d->icon.addPixmap(KWindowSystem::icon( d->win, KIconLoader::SizeMedium, KIconLoader::SizeMedium, false));
-    d->icon.addPixmap(KWindowSystem::icon( d->win, KIconLoader::SizeLarge, KIconLoader::SizeLarge, false));
+    if (d->icon.isNull()) {
+        d->icon.addPixmap(KWindowSystem::icon( d->win, KIconLoader::SizeSmall, KIconLoader::SizeSmall, false));
+        d->icon.addPixmap(KWindowSystem::icon( d->win, KIconLoader::SizeSmallMedium, KIconLoader::SizeSmallMedium, false));
+        d->icon.addPixmap(KWindowSystem::icon( d->win, KIconLoader::SizeMedium, KIconLoader::SizeMedium, false));
+        d->icon.addPixmap(KWindowSystem::icon( d->win, KIconLoader::SizeLarge, KIconLoader::SizeLarge, false));
+    }
 
     return d->icon;
 }
@@ -451,10 +347,10 @@ QPixmap Task::bestIcon( int size, bool &isStaticIcon )
   switch( size ) {
   case KIconLoader::SizeSmall:
     {
-      pixmap = icon( 16, 16, true  );
+      pixmap = icon(16, 16, true);
 
       // Icon of last resort
-      if( pixmap.isNull() ) {
+      if (pixmap.isNull()) {
         pixmap = KIconLoader::global()->loadIcon( "xorg",
                                                   KIconLoader::NoGroup,
                                                   KIconLoader::SizeSmall );
@@ -534,113 +430,11 @@ bool Task::idMatch( const QString& id1, const QString& id2 )
   return false;
 }
 
-void Task::move()
-{
-    bool on_current = d->info.isOnCurrentDesktop();
-
-    if (!on_current)
-    {
-        KWindowSystem::setCurrentDesktop(d->info.desktop());
-        KWindowSystem::forceActiveWindow(d->win);
-    }
-
-    if (d->info.isMinimized())
-    {
-        KWindowSystem::unminimizeWindow(d->win);
-    }
-
-    QRect geom = d->info.geometry();
-    QCursor::setPos(geom.center());
-
-    NETRootInfo ri(QX11Info::display(), NET::WMMoveResize);
-    ri.moveResizeRequest(d->win, geom.center().x(),
-                         geom.center().y(), NET::Move);
-}
-
-void Task::resize()
-{
-    bool on_current = d->info.isOnCurrentDesktop();
-
-    if (!on_current)
-    {
-        KWindowSystem::setCurrentDesktop(d->info.desktop());
-        KWindowSystem::forceActiveWindow(d->win);
-    }
-
-    if (d->info.isMinimized())
-    {
-        KWindowSystem::unminimizeWindow(d->win);
-    }
-
-    QRect geom = d->info.geometry();
-    QCursor::setPos(geom.bottomRight());
-
-    NETRootInfo ri(QX11Info::display(), NET::WMMoveResize);
-    ri.moveResizeRequest(d->win, geom.bottomRight().x(),
-                         geom.bottomRight().y(), NET::BottomRight);
-}
-
-void Task::setMaximized(bool maximize)
-{
-    KWindowInfo info = KWindowSystem::windowInfo(d->win, NET::WMState | NET::XAWMState | NET::WMDesktop);
-    bool on_current = info.isOnCurrentDesktop();
-
-    if (!on_current)
-    {
-        KWindowSystem::setCurrentDesktop(info.desktop());
-    }
-
-    if (info.isMinimized())
-    {
-        KWindowSystem::unminimizeWindow(d->win);
-    }
-
-    NETWinInfo ni(QX11Info::display(), d->win, QX11Info::appRootWindow(), NET::WMState);
-
-    if (maximize)
-    {
-        ni.setState(NET::Max, NET::Max);
-    }
-    else
-    {
-        ni.setState(0, NET::Max);
-    }
-
-    if (!on_current)
-    {
-        KWindowSystem::forceActiveWindow(d->win);
-    }
-}
-
 void Task::toggleMaximized()
 {
     setMaximized(!isMaximized());
 }
 
-void Task::restore()
-{
-    KWindowInfo info = KWindowSystem::windowInfo(d->win, NET::WMState | NET::XAWMState | NET::WMDesktop);
-    bool on_current = info.isOnCurrentDesktop();
-
-    if (!on_current)
-    {
-        KWindowSystem::setCurrentDesktop(info.desktop());
-    }
-
-    if( info.isMinimized())
-    {
-        KWindowSystem::unminimizeWindow(d->win);
-    }
-
-    NETWinInfo ni(QX11Info::display(), d->win, QX11Info::appRootWindow(), NET::WMState);
-    ni.setState(0, NET::Max);
-
-    if (!on_current)
-    {
-        KWindowSystem::forceActiveWindow( d->win );
-    }
-}
-
 void Task::setIconified(bool iconify)
 {
     if (iconify)
@@ -671,12 +465,6 @@ void Task::toggleIconified()
     setIconified(!isIconified());
 }
 
-void Task::close()
-{
-    NETRootInfo ri( QX11Info::display(), NET::CloseWindow );
-    ri.closeWindowRequest( d->win );
-}
-
 void Task::raise()
 {
 //    kDebug(1210) << "Task::raise(): " << name();
@@ -691,14 +479,27 @@ void Task::lower()
 
 void Task::activate()
 {
-//    kDebug(1210) << "Task::activate():" << name();
     WId w = d->win;
     if (!d->transientsDemandingAttention.isEmpty()) {
         WindowList::const_iterator it = d->transientsDemandingAttention.end();
         --it;
         w = *it;
+    } else if (!d->transients.isEmpty()) {
+        WindowList::const_iterator it = d->transients.end();
+        --it;
+        KWindowInfo info = KWindowSystem::windowInfo(*it, NET::WMState | NET::XAWMState | NET::WMDesktop);
+        //this is a work around for (at least?) kwin where a shaded transient will prevent the main
+        //window from being brought forward unless the transient is actually pulled forward, most
+        //easily reproduced by opening a modal file open/save dialog on an app then shading the file
+        //dialog and trying to bring the window forward by clicking on it in a tasks widget
+        //TODO: do we need to check all the transients for shaded?
+        if (info.valid(true) && (info.state() & NET::Shaded)) {
+            w = *it;
+        }
     }
 
+
+    //kDebug(1210) << "Task::activate():" << name() << d->win << w;
     KWindowSystem::forceActiveWindow(w);
 }
 
@@ -714,117 +515,36 @@ void Task::activateRaiseOrIconify()
     }
 }
 
-void Task::toDesktop(int desk)
-{
-    NETWinInfo ni(QX11Info::display(), d->win, QX11Info::appRootWindow(), NET::WMDesktop);
-    if (desk == 0) {
-        if (isOnAllDesktops()) {
-            ni.setDesktop(KWindowSystem::currentDesktop());
-            KWindowSystem::forceActiveWindow(d->win);
-        } else {
-            ni.setDesktop(NETWinInfo::OnAllDesktops);
-        }
-
-        return;
-    }
-
-    ni.setDesktop(desk);
-
-    if (desk == KWindowSystem::currentDesktop()) {
-        KWindowSystem::forceActiveWindow(d->win);
-    }
-}
-
 void Task::toCurrentDesktop()
 {
     toDesktop(KWindowSystem::currentDesktop());
 }
 
-void Task::setAlwaysOnTop(bool stay)
-{
-    NETWinInfo ni( QX11Info::display(), d->win, QX11Info::appRootWindow(), NET::WMState);
-    if(stay)
-        ni.setState( NET::StaysOnTop, NET::StaysOnTop );
-    else
-        ni.setState( 0, NET::StaysOnTop );
-}
-
 void Task::toggleAlwaysOnTop()
 {
     setAlwaysOnTop( !isAlwaysOnTop() );
 }
 
-void Task::setKeptBelowOthers(bool below)
-{
-    NETWinInfo ni(QX11Info::display(), d->win, QX11Info::appRootWindow(), NET::WMState);
-
-    if (below)
-    {
-        ni.setState(NET::KeepBelow, NET::KeepBelow);
-    }
-    else
-    {
-        ni.setState(0, NET::KeepBelow);
-    }
-}
-
 void Task::toggleKeptBelowOthers()
 {
     setKeptBelowOthers(!isKeptBelowOthers());
 }
 
-void Task::setFullScreen(bool fullscreen)
-{
-    NETWinInfo ni(QX11Info::display(), d->win, QX11Info::appRootWindow(), NET::WMState);
-
-    if (fullscreen)
-    {
-        ni.setState(NET::FullScreen, NET::FullScreen);
-    }
-    else
-    {
-        ni.setState(0, NET::FullScreen);
-    }
-}
-
 void Task::toggleFullScreen()
 {
     setFullScreen(!isFullScreen());
 }
 
-void Task::setShaded(bool shade)
-{
-    NETWinInfo ni( QX11Info::display(), d->win, QX11Info::appRootWindow(), NET::WMState);
-    if(shade)
-        ni.setState( NET::Shaded, NET::Shaded );
-    else
-        ni.setState( 0, NET::Shaded );
-}
-
 void Task::toggleShaded()
 {
     setShaded( !isShaded() );
 }
 
-void Task::publishIconGeometry(QRect rect)
+void Task::clearPixmapData()
 {
-    if (rect == d->iconGeometry)
-    {
-        return;
-    }
-
-    d->iconGeometry = rect;
-    NETWinInfo ni(QX11Info::display(), d->win, QX11Info::appRootWindow(), 0);
-    NETRect r;
-
-    if (rect.isValid())
-    {
-        r.pos.x = rect.x();
-        r.pos.y = rect.y();
-        r.size.width = rect.width();
-        r.size.height = rect.height();
-    }
-    ni.setIconGeometry(r);
+    d->lastIcon = QPixmap();
+    d->pixmap = QPixmap();
+    d->icon = QIcon();
 }
 
 void Task::addMimeData(QMimeData *mimeData) const
@@ -927,6 +647,11 @@ WId Task::idFromMimeData(const QMimeData *mimeData, bool *ok)
     return id;
 }
 
+Task::WindowProperties::WindowProperties(unsigned int netWinInfoProperties, unsigned int netWinInfoProperties2)
+  : netWindowInfoProperties(netWinInfoProperties), netWindowInfoProperties2(netWinInfoProperties2)
+{
+}
+
 } // TaskManager namespace
 
 #include "task.moc"
